"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _crypto = _interopRequireDefault(require("crypto"));

var _fileType = _interopRequireDefault(require("file-type"));

var _path = _interopRequireDefault(require("path"));

var _postcss = _interopRequireDefault(require("postcss"));

var _requestPromise = _interopRequireDefault(require("request-promise"));

var _chalk = _interopRequireDefault(require("chalk"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var urlReg = /url\(('|")?((?:http|\/\/)(?:[^"']+))(\1)\)/;

var getDeclUrl = value => {
  var url = value.match(urlReg)[2];

  var md5 = _crypto.default.createHash('md5');

  var urlIdentity = md5.update(url).digest('hex');
  return {
    urlIdentity,
    url
  };
};

var _default = _postcss.default.plugin('postcss-assets', function (_ref) {
  var {
    outputOptions,
    options
  } = _ref;
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  // 所有 css 中的网络请求
  var networkRequestMap = {};
  var publicPath = outputOptions.publicPath || '';
  var isUrlPublicPath = /^(https?\:)?\/\//.test(publicPath);
  return root => {
    return new Promise(resolve => {
      // 字体文件
      root.walkAtRules(atrule => {
        atrule.walkDecls(decl => {
          if (decl.prop == 'src') {
            decl.value.split(',').forEach(value => {
              if (urlReg.test(value)) {
                var {
                  url,
                  urlIdentity
                } = getDeclUrl(value);

                if (isUrlPublicPath && url.startsWith(publicPath)) {
                  // 已经是 publicPath 名下的网络资源可以不用本地化
                  return;
                }

                networkRequestMap[urlIdentity] = {
                  url,
                  decl
                };
              }
            });
          }
        });
      }); // 常规 css

      root.walkRules(rule => {
        rule.walkDecls(decl => {
          if (decl.prop == 'background-image' || decl.prop == 'background') {
            if (urlReg.test(decl.value)) {
              var {
                url,
                urlIdentity
              } = getDeclUrl(decl.value);

              if (isUrlPublicPath && url.startsWith(publicPath)) {
                // 已经是 publicPath 名下的网络资源可以不用本地化
                return;
              }

              networkRequestMap[urlIdentity] = {
                url,
                decl
              };
            }
          }
        });
      });

      if (Object.keys(networkRequestMap).length > 0) {
        Promise.all(Object.entries(networkRequestMap).map((_ref2) => {
          var [urlIdentity, networkRequest] = _ref2;
          var originUrl = networkRequest.url;
          var url = originUrl.startsWith('http') ? originUrl : "http:".concat(originUrl);
          return _requestPromise.default.get(_objectSpread({
            url,
            encoding: null
          }, options.requsetOptions)).then(res => {
            var buffer = Buffer.from(res, 'utf-8');

            var fileExtName = _path.default.extname(url);

            var fileExtType = (0, _fileType.default)(buffer);

            var md5 = _crypto.default.createHash('md5');

            var ext = fileExtType && fileExtType.ext ? ".".concat(fileExtType.ext) : fileExtName;
            var basename = md5.update(buffer).digest('hex') + ext;

            var contextPath = _path.default.join(options.relativeCssPath, options.outputPath, basename).replace(/\\/g, '/');

            var outputPath = _path.default.join(options.outputPath, basename).replace(/\\/g, '/');

            var publicFullPath = "".concat(outputOptions.publicPath).concat(outputPath);
            var asset = {
              contents: buffer,
              contextPath,
              outputPath,
              basename,
              publicFullPath
            };
            networkRequestMap[urlIdentity] = asset;
            opts.emit(asset);
            return Promise.resolve(asset);
          }).catch(err => {
            console.log(_chalk.default.cyan('[ExtractCssAssetsWebpackPlugin]'), _chalk.default.yellow('Warning:'), 'Asset download failed', _chalk.default.blue.underline(url));
            console.log('   ', err.error.toString());
            delete networkRequestMap[urlIdentity];
          });
        })).then(() => {
          // 字体文件
          root.walkAtRules(atrule => {
            atrule.walkDecls(decl => {
              if (decl.prop == 'src') {
                var newValue = decl.value.split(',').map(value => {
                  if (urlReg.test(value)) {
                    var {
                      urlIdentity,
                      url
                    } = getDeclUrl(value);

                    if (isUrlPublicPath && url.startsWith(publicPath)) {
                      // 已经是 publicPath 名下的网络资源可以不用本地化
                      return value;
                    }

                    return value.replace(urlReg, str => {
                      if (networkRequestMap[urlIdentity] && isUrlPublicPath && networkRequestMap[urlIdentity].publicFullPath) {
                        return "url('".concat(networkRequestMap[urlIdentity].publicFullPath, "')");
                      } else if (networkRequestMap[urlIdentity] && networkRequestMap[urlIdentity].contextPath) {
                        return "url('".concat(networkRequestMap[urlIdentity].contextPath, "')");
                      }

                      return str;
                    });
                  }

                  return value;
                }).join(',');
                decl.value = newValue;
              }
            });
          }); // 常规 css

          root.walkRules(rule => {
            rule.walkDecls(decl => {
              if (decl.prop == 'background-image' || decl.prop == 'background') {
                if (urlReg.test(decl.value)) {
                  var {
                    urlIdentity,
                    url
                  } = getDeclUrl(decl.value);

                  if (isUrlPublicPath && url.startsWith(publicPath)) {
                    // 已经是 publicPath 名下的网络资源可以不用本地化
                    return;
                  }

                  decl.value = decl.value.replace(urlReg, str => {
                    if (networkRequestMap[urlIdentity] && isUrlPublicPath && networkRequestMap[urlIdentity].publicFullPath) {
                      return "url('".concat(networkRequestMap[urlIdentity].publicFullPath, "')");
                    } else if (networkRequestMap[urlIdentity] && networkRequestMap[urlIdentity].contextPath) {
                      return "url('".concat(networkRequestMap[urlIdentity].contextPath, "')");
                    }

                    return str;
                  });
                }
              }
            });
          });
          resolve();
        });
      } else {
        resolve();
      }
    });
  };
});

exports.default = _default;